#!/usr/local/bin/perl
#
# This is a quick script for converting the log format into the format
# expected by dshield.org log reporting.  See http://www.dshield.org for
# more details.
#
# Written by Alec Kosky <aleckosky@hotmail.com>
#########################################################################

#
# Give the month table:
#
@months =
{
   "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct",
   "Nov", "Dec"
};

#
# We need to know our username, the timezone offset, and our IP address.
#
($user  = `whoami`) =~ s/\r*\n*$//;
($tzoff = `date +%z`) =~ s/^(\-\d{2})(\d{2})\r*\n*$/$1:$2/;
$ipaddr = "10.1.1.2";

#
# Ensure that we have the correct number of command-line arguments.
#
exit_usage()
   if ($ARGV[0] ne "-d" && $ARGV[0] ne "-s");

#
# Read the logs from STDIN, line at a time. 
#
while (<STDIN>)
{
   my ($flags);

   #
   # Get the TCP flags
   #
   ($flags = $_) =~ s/.*connection\s+\(([A-Z\|\s]+)\)\s+attempt\s+.*\r*\n*/$1/;
   $flags =~ s/(\s*([A-Z])[A-Z]{2}\s*\|*)/$2/g;

   $valid_line = 0;
   if ($ARGV[0] eq "-d")
   {
      #
      # This log is in multilog format (from the daemontools package).
      #
      if (/\d{2}\.\d+\s+port\s+/i)
      {
         $valid_line = 1;
         s/(\d{4}\-\d{2}\-\d{2})\s+(\d{2}:\d{2}:\d{2})\.\d+\s+port\s+(\d+)\s+connection\s+\(([A-Z\|\s]+)\)\s+attempt\s+from\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s+\([^\)]+\)\s+port\s+(\d+).*/$1 $2 $tzoff\t$user\t1\t$5\t$6\t$ipaddr\t$3\tTCP\t$flags/i
      }
      elsif (/\d{2}\.\d+\s+\w+\s+connection\s+/i)
      {
         $valid_line = 1;
         s/(\d{4}\-\d{2}\-\d{2})\s+(\d{2}:\d{2}:\d{2})\.\d+\s+(\w+)\s+connection\s+\(([A-Z\|\s]+)\)\s+attempt\s+from\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s+\([^\)]+\)\s+port\s+(\d+).*/$1 $2 $tzoff\t$user\t1\t$5\t$6\t$ipaddr\t$3\tTCP\t$flags/i
      }
   }
   elsif ($ARGV[0] eq "-s")
   {
      my ($datestr, $day, $i, $mo, $year);

      #
      # This log is in syslog format - we need to pull out the date and
      # convert it into YYYY-MM-DD format
      #
      ($year = `date +%Y`) =~ s/(.*)\r*\n*/$1/;
      ($datestr = $_) =~ s/^(\w{3}\s*\d{1,2}).*\r*\n*/$1/;
      ($mo, $day) = split /\s+/, $datestr;
      $day = "0$day"
         if ($day =~ /^\d{1}$/);
      for ($i = 0; $i < scalar(@months); $i++)
      {
         $mo = $months[$i]
            if ($datestr =~ /^$months[$i]/i);
      }
      $datestr = "$year-$mo-$day";

      if (/\[\d+\]:\s+port\s+/i)
      {
         $valid_line = 1;
         s/^(\w{3}\s*\d{1,2})\s+(\d{2}:\d{2}:\d{2})\s+\w+\s+\w+\[\d+\]:\s+port\s+(\d+)\s+connection\s+\(([A-Z\|\s]+)\)\s+attempt\s+from\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s+\([^\)]+\)\s+port\s+(\d+).*/$1 $2 $tzoff\t$user\t1\t$5\t$6\t$ipaddr\t$3\tTCP\t$flags/i
      }
      elsif (/\[\d+\]:\s+\w+\s+connection\s+/i)
      {
         $valid_line = 1;
         s/^(\w{3}\s*\d{1,2})\s+(\d{2}:\d{2}:\d{2})\s+\w+\s+\w+\[\d+\]:\s+(\w+)\s+connection\s+\(([A-Z\|\s]+)\)\s+attempt\s+from\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s+\([^\)]+\)\s+port\s+(\d+).*/$1 $2 $tzoff\t$user\t1\t$5\t$6\t$ipaddr\t$3\tTCP\t$flags/i
      }
   }

   #
   # If we have a valid line, print the output.
   #
   print $_
      if ($valid_line == 1);
}

#
# We're done processing the logs.  Get outta here.
#
exit (0);


sub exit_usage
{
   #
   # Print the usage message.
   #
   print <<_EOF_;
Usage: $0 argument
   Where argument is exactly one of the following:
      -h: Gives this help message
      -d: Assume that the logs are in D.J. Bernstein's multilog format
      -s: Assume that the logs are in syslog format

Sample Usage(s):

   Using syslog-formatted logs:
       cat log_file | $0 -s

   Using multilog-formatted logs:
      cat log_file | tai64nlocal | $0 -d

_EOF_

   #
   # Now exit.
   #
   exit (1);
}
