/*******************************************************************
  ignore.c  --  Copyright 1999 - 2003
                   Alec Kosky (aleckosky@hotmail.com)

Alright, here's the deal (i.e. the AK License):

You are free to use this program for commercial as well as 
   non-commercial endeavors.
You are free to modify this program to suit your needs.
You are free to redistribute unmodified copies of this program.

Basically, you can do what you like with it. However, you are not
permitted to redistribute modified copies of this program. If you
would like to see changes made to this program, please submit   
them to me (aleckosky@hotmail.com) and I will incorporate them into
future versions if I feel that they are beneficial.

********************************************************************/
#include "connlogd.h"

/*
 * This is the global host hash table
 */
struct ign_value host_list[HOST_HASH_SIZE];
struct int_value dport_list[DPORT_HASH_SIZE];

/*
 * Forward declarations.
 */
int s_port_clear(unsigned long int host);

/*
 * Clear all ignored hosts from memory.
 * Return 0 if successful, non-zero otherwise.
 */
int host_clear(void)
{
   struct ign_value  *tmp;
   int                i;

   for (i = 0; i < HOST_HASH_SIZE; i++)
   {
      s_port_clear(host_list[i].value);
      host_list[i].value = 0;
      host_list[i].sp_cnt = 0;
      while (host_list[i].next != NULL)
      {
         tmp = host_list[i].next;
         host_list[i].next = tmp->next;
         free(tmp);
      }
      host_list[i].next = NULL;
   }

   /*
    * We're done here.
    */
   return (0);
}

/*
` * This function will find a given host in the hash table
 * of hosts to be ignored. Note: The host is stored by
 * the IP address value, not the xxx.xxx.xxx.xxx string.
 *
 * Will return a pointer to the structure that contains all of
 * the relevant host information if it is found in the table,
 * otherwise it will return NULL.
 */
struct ign_value *host_find(unsigned long int host)
{
   struct ign_value   *tmp;
   unsigned long int   h_entry;

   h_entry = host % HOST_HASH_SIZE;

   /*
    * Check to see if the hash entry is empty.
    */
   if (host_list[h_entry].value == host)
      tmp = (struct ign_value *)&host_list[h_entry];
   else
   {
      /*
       * Well, the hash entry wasn't empty, and it wasn't the
       * entry that we were looking for. Let's now go through
       * the linked list to see if it's there.
       */
      tmp = host_list[h_entry].next;
      while (tmp != NULL)
      {
         if (tmp->value == host)
            break;
         tmp = tmp->next;
      }
   }


   /*
    * Return the tmp pointer to the calling function.
    */
   return (tmp);
}

int host_insert(unsigned long int host)
{
   struct ign_value   *tmp;
   unsigned long int   h_entry;
   int                 i;

   h_entry = host % HOST_HASH_SIZE;

   /*
    * Check to see if the hash entry is empty.
    */
   if (host_list[h_entry].value == 0)
   {
      host_list[h_entry].value  = host;
      host_list[h_entry].sp_cnt = 0;
      host_list[h_entry].next   = NULL;
      for (i = 0; i < SPORT_HASH_SIZE; i++)
      {
         host_list[h_entry].port_list[i].value = 0;
         host_list[h_entry].port_list[i].next  = NULL;
      }
   }
   else if (host_list[h_entry].value == host)
      return (1);	/* already exists! */
   else
   {
      /*
       * Well, the hash entry wasn't empty, and it wasn't the
       * entry that we were looking for. Let's now go through
       * the linked list to see if it's there.
       */
      tmp = host_list[h_entry].next;
      while (tmp != NULL)
      {
         if (tmp->next == NULL)
            break;
         tmp = tmp->next;
      }

      /*
       * Attempt to allocate space for a new host on the hash.
       */
      if (tmp == NULL)
      {
         if ((host_list[h_entry].next =
              (struct ign_value *)calloc(1, sizeof(struct ign_value))) == NULL)
            return (-1);
         else
         {
            host_list[h_entry].next->value  = host;
            host_list[h_entry].next->sp_cnt = 0;
            host_list[h_entry].next->next   = NULL;
            for (i = 0; i < SPORT_HASH_SIZE; i++)
            {
               host_list[h_entry].next->port_list[i].value = 0;
               host_list[h_entry].next->port_list[i].next  = NULL;
            }
         }
      }
      else
      {
         if ((tmp->next =
              (struct ign_value *)calloc(1, sizeof(struct ign_value))) == NULL)
            return (-1);
         else
         {
            tmp->next->value  = host;
            tmp->next->sp_cnt = 0;
            tmp->next->next   = NULL;
            for (i = 0; i < SPORT_HASH_SIZE; i++)
            {
               tmp->next->port_list[i].value = 0;
               tmp->next->port_list[i].next  = NULL;
            }
         }
      }
   }


   /*
    * Let 'em know that things were OK.
    */
   return (0);
}

/*
 * Clear all of the ignored source ports for the given host from memory.
 * Return 0 if successful, non-zero otherwise.
 */
int s_port_clear(unsigned long int host)
{
   struct ign_value   *h;
   struct int_value   *tmp;
   int                 i;

   if ((h = host_find(host)) == NULL)
      return (-1);

   for (i = 0; i < SPORT_HASH_SIZE; i++)
   {
      h->port_list[i].value = 0;
      while (h->port_list[i].next != NULL)
      {
         tmp = h->port_list[i].next;
         h->port_list[i].next = tmp->next;
         free(tmp);
      }
      h->sp_cnt = 0;
   }

   /*
    * We're outta here.
    */
   return (0);
}

/*
 * This is the function to find a given port in the hash table
 * containing the list of source ports that are to be ignored.
 * If there are no ports in the list, then all ports should be
 * ignored.
 *
 * It will return 0 upon success (i.e. the port is in the list
 * and should be ignored), otherwise it will return a non-zero
 * value upon failure (the port was not in the list).
 */
int s_port_find(unsigned long int host, unsigned int port)
{
   struct ign_value   *h;
   struct int_value   *v;
   unsigned int        h_entry;

   /*
    * Can we find the host?
    */
   if ((h = host_find(host)) == NULL)
      return (-1);

   /*
    * If there were no ports specified in the configuration file,
    * then all ports are to be ignored (unless there were no
    * ports specified for all hosts [*], then we return -1 to
    * indicate that the connection is not to be ignored).
    */
   if (h->value == 0 && h->sp_cnt == 0)
      return (-1);
   else if (h->sp_cnt == 0)
      return (0);

   h_entry = port % SPORT_HASH_SIZE;
   if (h->port_list[h_entry].value == 0)
      return (-1);
   else if (h->port_list[h_entry].value == port)
      return (0);
   else
   {
      /*
       * We now have a linear search on our hands - let's
       * run through each value in the linked list.
       */
      v = h->port_list[h_entry].next;
      while (v != NULL)
      {
         if (v->value == host)
            return (0);
         v = v->next;
      }
   }

   return (-1);
}

int s_port_insert(unsigned long int host, unsigned int port)
{
   struct ign_value   *h;
   struct int_value   *v;
   unsigned int        h_entry;

   if ((h = host_find(host)) == NULL)
      return (-2);

   h_entry = port % SPORT_HASH_SIZE;
   if (h->port_list[h_entry].value == 0)
   {
      h->port_list[h_entry].value = port;
      h->port_list[h_entry].next  = NULL;
      h->sp_cnt++;
   }
   else if (h->port_list[h_entry].value == port)
      return (1);	/* already exists!?!? */
   else
   {
      /*
       * We now have a linear search on our hands - let's
       * run through each value in the linked list.
       */
      v = h->port_list[h_entry].next;
      while (v != NULL)
      {
         if (v->next == NULL)
            break;
         v = v->next;
      }

      /*
       * Attempt to allocate space for a new port on the hash.
       */
      if (v == NULL)
      {
         if ((h->port_list[h_entry].next =
              (struct int_value *)calloc(1, sizeof(struct int_value))) == NULL)
            return (-1);
         else
         {
            h->port_list[h_entry].next->value = port;
            h->port_list[h_entry].next->next  = NULL;
         }
      }
      else
      {
         if ((v->next =
              (struct int_value *)calloc(1, sizeof(struct int_value))) == NULL)
            return (-1);
         else
         {
            v->next->value = port;
            v->next->next  = NULL;
         }
      }

      /*
       * Increment the number of source ports in the host list.
       */
      h->sp_cnt++;
   }

   return (0);
}

/*
 * Clear all of the ignored destination ports from memory.
 * Return 0 if successful, non-zero otherwise.
 */
int d_port_clear(void)
{
   struct int_value   *tmp;
   int                 i;

   for (i = 0; i < DPORT_HASH_SIZE; i++)
   {
      dport_list[i].value = 0;
      while (dport_list[i].next != NULL)
      {
         tmp = dport_list[i].next;
         dport_list[i].next = tmp->next;
         free(tmp);
      }
   }

   return (0);
}

/*
 * This is the function to find a given port in the hash table
 * containing the list of destination ports that are to be ignored.
 *
 * It will return 0 upon success (i.e. the port is in the list
 * and should be ignored), otherwise it will return a non-zero
 * value upon failure (the port was not in the list).
 */
int d_port_find(unsigned int port)
{
   struct int_value   *tmp;
   unsigned int        h_entry;

   h_entry = port % DPORT_HASH_SIZE;

   /*
    * Check to see if the hash entry is empty.
    */
   if (dport_list[h_entry].value == 0)
      return (-1);
   else if (dport_list[h_entry].value == port)
      return (0);
   else
   {
      /*
       * Well, the hash entry wasn't empty, and it wasn't the
       * entry that we were looking for. Let's now go through
       * the linked list to see if it's there.
       */
      tmp = dport_list[h_entry].next;
      while (tmp != NULL)
      {
         if (tmp->value == port)
            return (0);
         tmp = tmp->next;
      }
   }


   /*
    * Get outta here - nothing found.
    */
   return (-1);
}

int d_port_insert(unsigned int port)
{
   struct int_value   *tmp;
   unsigned int        h_entry;

   h_entry = port % DPORT_HASH_SIZE;

   /*
    * Check to see if the hash entry is empty.
    */
   if (dport_list[h_entry].value == 0)
   {
      dport_list[h_entry].value = port;
      dport_list[h_entry].next  = NULL;
   }
   else if (dport_list[h_entry].value == port)
      return (1);
   else
   {
      /*
       * Well, the hash entry wasn't empty, and it wasn't the
       * entry that we were looking for. Let's now go through
       * the linked list to find the end.
       */
      tmp = dport_list[h_entry].next;
      while (tmp != NULL)
      {
         if (tmp->next == NULL)
            break;
         tmp = tmp->next;
      }

      /*
       * Attempt to allocate space for a new port on the hash.
       */
      if (tmp == NULL)
      {
         if ((dport_list[h_entry].next =
              (struct int_value *)calloc(1, sizeof(struct int_value))) == NULL)
            return (-1);
         else
         {
            dport_list[h_entry].next->value = port;
            dport_list[h_entry].next->next  = NULL;
         }
      }
      else
      {
         if ((tmp->next =
              (struct int_value *)calloc(1, sizeof(struct int_value))) == NULL)
            return (-1);
         else
         {
            tmp->next->value = port;
            tmp->next->next  = NULL;
         }
      }
      
   }


   /*
    * Get outta here - nothing found.
    */
   return (0);
}


